@@ -1,22 +1,20 @@
 import inspect
-import sys
 import threading
 import time
 import traceback
 from concurrent.futures import ThreadPoolExecutor
-from io import StringIO
-from typing import Any
+from typing import List
 
 import ophyd
-from bec_lib.core import Alarms, BECMessage, BECService, MessageEndpoints, bec_logger
-from bec_lib.core.BECMessage import BECStatus
-from bec_lib.core.connector import ConnectorBase
-from bec_lib.core.devicemanager import OnFailure
+from bec_lib import Alarms, BECService, MessageEndpoints, bec_logger, messages
+from bec_lib.connector import ConnectorBase
+from bec_lib.devicemanager import OnFailure
+from bec_lib.messages import BECStatus
 from ophyd import Staged
 from ophyd.utils import errors as ophyd_errors
 
-from device_server.devices import is_serializable, rgetattr
 from device_server.devices.devicemanager import DeviceManagerDS
+from device_server.rpc_mixin import RPCMixin
 
 logger = bec_logger.logger
 
@@ -24,14 +22,14 @@ consumer_stop = threading.Event()
 
 
 class DisabledDeviceError(Exception):
-    pass
+    """Exception raised when a disabled device is accessed"""
 
 
 class InvalidDeviceError(Exception):
-    pass
+    """Exception raised when an invalid device is accessed"""
 
 
-class DeviceServer(BECService):
+class DeviceServer(RPCMixin, BECService):
     """DeviceServer using ophyd as a service
     This class is intended to provide a thin wrapper around ophyd and the devicemanager. It acts as the entry point for other services
     """
@@ -73,6 +71,7 @@ class DeviceServer(BECService):
         self.status = BECStatus.RUNNING
 
     def update_status(self, status: BECStatus):
+        """update the status of the device server"""
         self.status = status
 
     def stop(self) -> None:
@@ -100,7 +99,7 @@ class DeviceServer(BECService):
     @staticmethod
     def consumer_interception_callback(msg, *, parent, **_kwargs) -> None:
         """callback for receiving scan modifications / interceptions"""
-        mvalue = BECMessage.ScanQueueModificationMessage.loads(msg.value)
+        mvalue = messages.ScanQueueModificationMessage.loads(msg.value)
         if mvalue is None:
             logger.warning("Failed to parse scan queue modification message.")
             return
@@ -120,7 +119,7 @@ class DeviceServer(BECService):
                 dev.obj.stop()
         self.status = BECStatus.RUNNING
 
-    def _assert_device_is_enabled(self, instructions: BECMessage.DeviceInstructionMessage) -> None:
+    def _assert_device_is_enabled(self, instructions: messages.DeviceInstructionMessage) -> None:
         devices = instructions.content["device"]
 
         if isinstance(devices, str):
@@ -130,7 +129,7 @@ class DeviceServer(BECService):
             if not self.device_manager.devices[dev].enabled:
                 raise DisabledDeviceError(f"Cannot access disabled device {dev}.")
 
-    def _assert_device_is_valid(self, instructions: BECMessage.DeviceInstructionMessage) -> None:
+    def _assert_device_is_valid(self, instructions: messages.DeviceInstructionMessage) -> None:
         devices = instructions.content["device"]
         if not devices:
             raise InvalidDeviceError("At least one device must be specified.")
@@ -150,7 +149,7 @@ class DeviceServer(BECService):
         """
         action = None
         try:
-            instructions = BECMessage.DeviceInstructionMessage.loads(msg)
+            instructions = messages.DeviceInstructionMessage.loads(msg)
             if not instructions.content["device"]:
                 return
             action = instructions.content["action"]
@@ -165,7 +164,7 @@ class DeviceServer(BECService):
             elif action == "read":
                 self._read_device(instructions)
             elif action == "rpc":
-                self._run_rpc(instructions)
+                self.run_rpc(instructions)
             elif action == "kickoff":
                 self._kickoff_device(instructions)
             elif action == "complete":
@@ -210,104 +209,7 @@ class DeviceServer(BECService):
         """callback for handling device instructions"""
         parent.executor.submit(parent.handle_device_instructions, msg.value)
 
-    def _get_result_from_rpc(self, rpc_var: Any, instr_params: dict) -> Any:
-        if callable(rpc_var):
-            args = tuple(instr_params.get("args", ()))
-            kwargs = instr_params.get("kwargs", {})
-            if args and kwargs:
-                res = rpc_var(*args, **kwargs)
-            elif args:
-                res = rpc_var(*args)
-            elif kwargs:
-                res = rpc_var(**kwargs)
-            else:
-                res = rpc_var()
-        else:
-            res = rpc_var
-        if not is_serializable(res):
-            if isinstance(res, ophyd.StatusBase):
-                return res
-            elif isinstance(res, list) and instr_params.get("func") in ["stage", "unstage"]:
-                return [obj._staged for obj in res]
-            res = None
-            self.connector.raise_alarm(
-                severity=Alarms.WARNING,
-                alarm_type="TypeError",
-                source=instr_params,
-                content=f"Return value of rpc call {instr_params} is not serializable.",
-                metadata={},
-            )
-        return res
-
-    def _run_rpc(self, instr: BECMessage.DeviceInstructionMessage) -> None:
-        save_stdout = sys.stdout
-        result = StringIO()
-        sys.stdout = result
-        try:
-            instr_params = instr.content.get("parameter")
-            self._assert_device_is_enabled(instr)
-            rpc_var = rgetattr(
-                self.device_manager.devices[instr.content["device"]].obj,
-                instr_params.get("func"),
-            )
-            res = self._get_result_from_rpc(rpc_var, instr_params)
-            if isinstance(res, ophyd.StatusBase):
-                res.__dict__["instruction"] = instr
-                res.add_callback(self._status_callback)
-                res = {
-                    "type": "status",
-                    "RID": instr.metadata.get("RID"),
-                    "success": res.success,
-                    "timeout": res.timeout,
-                    "done": res.done,
-                    "settle_time": res.settle_time,
-                }
-            elif isinstance(res, list) and isinstance(res[0], ophyd.Staged):
-                res = [str(stage) for stage in res]
-            else:
-                print(f"return value: {res}")
-            # send result to client
-            self.producer.set(
-                MessageEndpoints.device_rpc(instr_params.get("rpc_id")),
-                BECMessage.DeviceRPCMessage(
-                    device=instr.content["device"],
-                    return_val=res,
-                    out=result.getvalue(),
-                    success=True,
-                ).dumps(),
-                expire=1800,
-            )
-            logger.trace(res)
-        except KeyboardInterrupt as kbi:
-            sys.stdout = save_stdout
-            raise KeyboardInterrupt from kbi
-
-        except Exception as exc:  # pylint: disable=broad-except
-            # send error to client
-            self._send_rpc_exception(exc, instr)
-
-        finally:
-            sys.stdout = save_stdout
-
-    def _send_rpc_exception(self, exc: Exception, instr: BECMessage.DeviceInstructionMessage):
-        exc_formatted = {
-            "error": exc.__class__.__name__,
-            "msg": exc.args,
-            "traceback": traceback.format_exc(),
-        }
-        logger.info(f"Received exception: {exc_formatted}, {exc}")
-        instr_params = instr.content.get("parameter")
-        self.producer.set(
-            MessageEndpoints.device_rpc(instr_params.get("rpc_id")),
-            BECMessage.DeviceRPCMessage(
-                device=instr.content["device"],
-                return_val=None,
-                out=exc_formatted,
-                success=False,
-            ).dumps(),
-        )
-
-    def _trigger_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+    def _trigger_device(self, instr: messages.DeviceInstructionMessage) -> None:
         logger.debug(f"Trigger device: {instr}")
         devices = instr.content["device"]
         if not isinstance(devices, list):
@@ -319,7 +221,7 @@ class DeviceServer(BECService):
             status.__dict__["instruction"] = instr
             status.add_callback(self._status_callback)
 
-    def _kickoff_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+    def _kickoff_device(self, instr: messages.DeviceInstructionMessage) -> None:
         logger.debug(f"Kickoff device: {instr}")
         obj = self.device_manager.devices.get(instr.content["device"]).obj
         kickoff_args = inspect.getfullargspec(obj.kickoff).args
@@ -332,7 +234,7 @@ class DeviceServer(BECService):
         status.__dict__["instruction"] = instr
         status.add_callback(self._status_callback)
 
-    def _complete_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+    def _complete_device(self, instr: messages.DeviceInstructionMessage) -> None:
         if instr.content["device"] is None:
             devices = [dev.name for dev in self.device_manager.devices.enabled_devices]
         else:
@@ -352,7 +254,7 @@ class DeviceServer(BECService):
             status.__dict__["instruction"] = instr
             status.add_callback(self._status_callback)
 
-    def _set_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+    def _set_device(self, instr: messages.DeviceInstructionMessage) -> None:
         device_obj = self.device_manager.devices.get(instr.content["device"])
         if not device_obj.enabled_set:
             raise DisabledDeviceError(
@@ -366,7 +268,7 @@ class DeviceServer(BECService):
         status.__dict__["instruction"] = instr
         status.add_callback(self._status_callback)
 
-    def _pre_scan(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+    def _pre_scan(self, instr: messages.DeviceInstructionMessage) -> None:
         devices = instr.content["device"]
         if not isinstance(devices, list):
             devices = [devices]
@@ -376,7 +278,7 @@ class DeviceServer(BECService):
             obj.metadata = instr.metadata
             if hasattr(obj.obj, "pre_scan"):
                 obj.obj.pre_scan()
-            dev_msg = BECMessage.DeviceReqStatusMessage(
+            dev_msg = messages.DeviceReqStatusMessage(
                 device=dev,
                 success=True,
                 metadata=instr.metadata,
@@ -390,7 +292,7 @@ class DeviceServer(BECService):
             device_name = status.device.root.name
         else:
             device_name = status.obj.root.name
-        dev_msg = BECMessage.DeviceReqStatusMessage(
+        dev_msg = messages.DeviceReqStatusMessage(
             device=device_name,
             success=status.success,
             metadata=status.instruction.metadata,
@@ -410,19 +312,24 @@ class DeviceServer(BECService):
 
         pipe.execute()
 
-    def _read_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+    def _read_device(self, instr: messages.DeviceInstructionMessage) -> None:
         # check performance -- we might have to change it to a background thread
         devices = instr.content["device"]
         if not isinstance(devices, list):
             devices = [devices]
 
+        self._read_and_update_devices(devices, instr.metadata)
+
+    def _read_and_update_devices(self, devices: List[str], metadata: dict) -> list:
         start = time.time()
         pipe = self.producer.pipeline()
+        signal_container = []
         for dev in devices:
-            self.device_manager.devices.get(dev).metadata = instr.metadata
+            self.device_manager.devices.get(dev).metadata = metadata
             obj = self.device_manager.devices.get(dev).obj
             try:
                 signals = obj.read()
+                signal_container.append(signals)
             except Exception as exc:
                 self.device_manager.connector.raise_alarm(
                     severity=Alarms.WARNING,
@@ -441,7 +348,7 @@ class DeviceServer(BECService):
                 elif ds_dev.on_failure == OnFailure.BUFFER:
                     # if possible, fall back to past readings
                     logger.warning(f"Failed to read {dev}. Trying to load an old value.")
-                    old_msg = BECMessage.DeviceMessage.loads(
+                    old_msg = messages.DeviceMessage.loads(
                         self.producer.get(MessageEndpoints.device_read(dev))
                     )
                     if not old_msg:
@@ -450,27 +357,26 @@ class DeviceServer(BECService):
 
             self.producer.set_and_publish(
                 MessageEndpoints.device_read(dev),
-                BECMessage.DeviceMessage(signals=signals, metadata=instr.metadata).dumps(),
+                messages.DeviceMessage(signals=signals, metadata=metadata).dumps(),
                 pipe,
             )
             self.producer.set_and_publish(
                 MessageEndpoints.device_readback(dev),
-                BECMessage.DeviceMessage(signals=signals, metadata=instr.metadata).dumps(),
+                messages.DeviceMessage(signals=signals, metadata=metadata).dumps(),
                 pipe,
             )
             self.producer.set(
                 MessageEndpoints.device_status(dev),
-                BECMessage.DeviceStatusMessage(
-                    device=dev, status=0, metadata=instr.metadata
-                ).dumps(),
+                messages.DeviceStatusMessage(device=dev, status=0, metadata=metadata).dumps(),
                 pipe,
             )
         pipe.execute()
         logger.debug(
             f"Elapsed time for reading and updating status info: {(time.time()-start)*1000} ms"
         )
+        return signal_container
 
-    def _stage_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+    def _stage_device(self, instr: messages.DeviceInstructionMessage) -> None:
         devices = instr.content["device"]
         if not isinstance(devices, list):
             devices = [devices]
@@ -486,14 +392,12 @@ class DeviceServer(BECService):
                 self.device_manager.devices[dev].obj.stage()
             self.producer.set(
                 MessageEndpoints.device_staged(dev),
-                BECMessage.DeviceStatusMessage(
-                    device=dev, status=1, metadata=instr.metadata
-                ).dumps(),
+                messages.DeviceStatusMessage(device=dev, status=1, metadata=instr.metadata).dumps(),
                 pipe,
             )
         pipe.execute()
 
-    def _unstage_device(self, instr: BECMessage.DeviceInstructionMessage) -> None:
+    def _unstage_device(self, instr: messages.DeviceInstructionMessage) -> None:
         devices = instr.content["device"]
         if not isinstance(devices, list):
             devices = [devices]
@@ -509,9 +413,7 @@ class DeviceServer(BECService):
                     logger.debug(f"Device {obj.name} was already unstaged.")
             self.producer.set(
                 MessageEndpoints.device_staged(dev),
-                BECMessage.DeviceStatusMessage(
-                    device=dev, status=0, metadata=instr.metadata
-                ).dumps(),
+                messages.DeviceStatusMessage(device=dev, status=0, metadata=instr.metadata).dumps(),
                 pipe,
             )
         pipe.execute()
